#! /usr/bin/perl
# Usage:
#     change_stuff.perl
# Where input is from STDIN and output is to STDOUT. Thus, a typical
# invocation would be
#    change_stuff.perl < /path/to/in/file > /path/to/out/file
# or
#    change_stuff.perl < file > file.changed
#
# To use on multiple files, use a loop from the shell
#    for VAR in FILES ; do change_stuff.perl < $VAR > OUTSPEC ; done
# Where OUTSPEC must contain "$VAR" or you get one big file (which is
# likely not what you want). So, for all files in the current working
# directory (or on all marked files in your Emacs dired buffer), issue
#    for f in * ; do change_stuff.perl < $f > $f.fixed ; done
# To experiment only on XML files that start with "b", use
#    for f in b*.xml ; do change_stuff.perl < $f > /tmp/erase.me/$f ; done

# Command line tool to change all occurrences of "foo" to "bar" in the
# files specified. Obviously you replace "foo" and "bar" in the line:
# MEAT_OF_THE_MATTER: s/foo/bar/g;
# with the values you need.
#
# Updated 2003-04-29 by Syd Bauman to process entire files out once, rather
# than a line at a time.

use English;

@in = <STDIN>;			# read whole file into one array
$file = join '', @in;		# make it into one big scalar
undef @in;			# don't need this anymore

# now come the substitutions

# EXAMPLE: $file =~ s|<tei:ptr\starget="|<tei:ptr corresp="|g;

# initial data clean: delete erroneous extra whitespace
$file =~ s|(\@[a-z])[\s]+\\|\1 \\|g;

# first we need to deal with curly braces in the sub-heading; their use here now deprecated, so strip them out
$file =~ s|(\@w\\[^{]+){|\1UUUUU|g;
$file =~ s|UUUUU([^}]+)}|\1|g;

# next we deal with curly braces in footnotes
$file =~ s|(\@f\\[^{]+){|\1UUUUU|g;
$file =~ s|UUUUU([^}]+)}|<hi rend="italic">\1</hi>|g;

# next we deal with curly braces marking {blank}
$file =~ s|{blank}|<space/>|g;

# PROBLEM:  here we should take care of curly braces marking {signed}, but nothing marks where the end tag should go
# so this will have to be adjusted by hand
$file =~ s|{signed}|<signed></signed>|g;
# by this point any remaining curly braces should mark editorial expansions
$file =~ s|{|<ex>|g;
$file =~ s|}|</ex>|g;

# deal with ampersands and various special characters in the text
$file =~ s|&|&#x0026;|g;
$file =~ s|\@ae|&#x00E6;|g;
$file =~ s|\@AE|&#x00C6;|g;
$file =~ s|\@C|&#x204C;|g;
$file =~ s|\@d|&#x00F0;|g;
$file =~ s|\@n|&#x014A;|g;
$file =~ s|\@N|&#x014B;|g;
$file =~ s|\@oe|&#x0152;|g;
$file =~ s|\@OE|&#x0153;|g;
$file =~ s|\@P|&#x204B;|g;
$file =~ s|\@th|&#x00FE;|g;
$file =~ s|\@TH|&#x00DE;|g;
$file =~ s|\@v|&#x1EFD;|g;
$file =~ s|\@y|&#x01BF;|g;
$file =~ s|\@z|&#x021D;|g;
$file =~ s|\@Z|&#x021C;|g;
$file =~ s|\@\$|&#x00A3;|g;
$file =~ s|\@'a|&#x00E1;|g;
$file =~ s|\@'e|&#x00E9;|g;
$file =~ s|\@'i|&#x00ED;|g;
$file =~ s|\@'o|&#x00F3;|g;
$file =~ s|\@'u|&#x00FA;|g;
$file =~ s|\@'A|&#x00C1;|g;
$file =~ s|\@'E|&#x00C9;|g;
$file =~ s|\@'I|&#x00CD;|g;
$file =~ s|\@'O|&#x00D3;|g;
$file =~ s|\@'U|&#x00DA;|g;
$file =~ s|\@,a|&#x00E0;|g;
$file =~ s|\@,e|&#x00E8;|g;
$file =~ s|\@,i|&#x00EC;|g;
$file =~ s|\@,o|&#x00F2;|g;
$file =~ s|\@,u|&#x00F9;|g;
$file =~ s|\@,A|&#x00C0;|g;
$file =~ s|\@,E|&#x00C8;|g;
$file =~ s|\@,I|&#x00CC;|g;
$file =~ s|\@,O|&#x00D2;|g;
$file =~ s|\@,U|&#x00D9;|g;
$file =~ s|\@\^a|&#x00E2;|g;
$file =~ s|\@\^e|&#x00EA;|g;
$file =~ s|\@\^i|&#x00EE;|g;
$file =~ s|\@\^o|&#x00F4;|g;
$file =~ s|\@\^u|&#x00FB;|g;
$file =~ s|\@\^A|&#x00C2;|g;
$file =~ s|\@\^E|&#x00CA;|g;
$file =~ s|\@\^I|&#x00CE;|g;
$file =~ s|\@\^O|&#x00D4;|g;
$file =~ s|\@\^U|&#x00D8;|g;
$file =~ s|\@\?c|&#x00E7;|g;
$file =~ s|\@;|&punctelev;|g;
$file =~ s|\@\*|&#x00B7;|g;
$file =~ s|\@!|&#x0021;|g;
$file =~ s|\^|&#x005B;|g;
$file =~ s|--|&#x2013;|g;


$file =~ s|\@a\\|<add rend="above">|g;
$file =~ s|\@a\s\\|</add>|g;
$file =~ s|\@b\\|<add rend="below">|g;
$file =~ s|\@b\s\\|</add>|g;
$file =~ s|\@c\\|<note type="collation" n="CHANGE_ME_TO_XMLID">|g;
$file =~ s|\@c\s\\|</note>|g;
$file =~ s|\@e\\|<hi rend="bold">|g;
$file =~ s|\@e\s\\|</hi>|g;
$file =~ s|\@f\\|<note type="foot" n="CHANGE_ME_TO_XMLID">|g;
$file =~ s|\@f\s\\|</note>|g;
$file =~ s|\@g\\|<p rend="exdented">|g;
$file =~ s|\@g\s\\|</p>|g;
$file =~ s|\@i\\|<add><handShift/>|g;
$file =~ s|\@i\s\\|</add>|g;
$file =~ s|\@j\\|<hi rend="bold_italic>|g;
$file =~ s|\@j\s\\|</hi>|g;
# @k won't be used in record headings any more, so now just a rendition encoding
$file =~ s|\@k\\|<hi rend="smallcaps">|g;
$file =~ s|\@k\s\\|</hi>|g;
$file =~ s|\@l\\|<note type="marginale" place="marginLeft" n="CHANGE_ME_TO_XMLID">|g;
$file =~ s|\@l\s\\|</note>|g;
$file =~ s|\@m\\|<hi rend="center">|g;
$file =~ s|\@m\s\\|</hi>|g;
$file =~ s|\@p\\|<p rend="indented">|g;
$file =~ s|\@p\s\\|</p>|g;
# keep following line to deal with any instances of plain @p
$file =~ s|\@p|&#x0020;|g;
# q won't be used in record headings any more, so now just a rendition encoding
$file =~ s|\@q\\|<hi rend="smallcaps_italic">|g;
$file =~ s|\@q\s\\|</hi>|g;
$file =~ s|\@r\\|<note type="marginale" place="marginRight" n="CHANGE_ME_TO_XMLID">|g;
$file =~ s|\@r\s\\|</note>|g;
$file =~ s|\@s\\|<hi rend="superscript">|g;
$file =~ s|\@s\s\\|</hi>|g;
$file =~ s|\@x\\|<note type="eccles_court">|g;
$file =~ s|\@x\s\\|</note>|g;
# USE OF <t#> FOR TEXT IN COLUMNAR FORMAT HAS BEEN DEPRECATED - NEED MANUAL ENCODING OF <TABLE>
$file =~ s|<t#>|<!-- REEDFIX: NEED A TABLE HERE -->|g;
# square brackets: first we handle instances preceded by "@"
# USE OF @[ TO INDICATE FLUSH RIGHT SUMS IN ACCOUNTS HAS BEEN DEPRECATED - NEED MANUAL ENCODING OF <TABLE>
$file =~ s|\@\[|<!-- REEDFIX: NEED A TABLE HERE -->|g;
# remaining instances of square brackets should now all be marking cancellations
$file =~ s|\[([^\]]+)\]|<del>\1</del>|g;
$file =~ s|<\.>|<damage><gap unit="chars" quantity="1"/></damage>|g;
$file =~ s|<\.\.>|<damage><gap unit="chars" quantity="2"/></damage>|g;
$file =~ s|<\.\.\.>|<damage><gap unit="chars" quantity="3"/></damage>|g;
$file =~ s|\.\.\.|<gap/>|g;
# USE OF | TO INDICATE CHANGE OF FOLIO, PAGE, ETC. HAS BEEN DEPRECATED - THIS SHOULD HAVE BEEN DONE IN PALEOGRAPHIC CHECKING STAGE
# $file =~ s/\|/<pb type="" n=""\/>/g;

# SUB-HEADING
$file =~ s|\@w\\([^\\]+)\\!|<head type="sub">\1</head>|g;

# MAIN HEADINGS - 
# isolate and tag the place, tag beginning of date
 $file =~ s|\@h\\([^!]+)!|<head type="main"><seg type="REPLACE_ME_FROM_CODE_LIST" subtype="place_name">\1</seg> <date>QQQQQ|g;
# end the date, mark beginning of title heading
 $file =~ s|QQQQQ([^!]+)!|\1</date>ZZZZZ|g;
# end title heading
 $file =~ s|ZZZZZ([^\\]+)\\!|<seg type="REPLACE_ME_FROM_CODE_LIST" subtype="source_info">\1</seg></head>|g;



$file =~ s|!|<lb/>|g;

$file =~ s|||g;

print $file;			# print out the result

exit;

__DATA__


